;  OPT_SC_BCN.ASM.      G4JNT   Aug 2011

;32 bit / 8 bit DDS for Audio generation   
;Interrupt runs at Fosc / 160 (Fclock / 40).  /2 Presacle, int counts 20
;  Increments 32 bit counte rD0-3   with value in F0-3.
;  MS Byte used as entry to 8 bit sine lookup table
;Derived from FreqSource, RS232 removed, auto FSK added

    LIST P = 16F628A
    INCLUDE "P16F628A.INC"

    __config    0x3F62      ;LV Programming off , OSC HS Mode   use 3F63 for ext source, 3F61 XT

    __idlocs    0x8130

	errorlevel 2

    org	0x2100

FREQ	de	0x53, 0x55, 0x55, 0x55		;25kHz based on 76.8kHz clock
	de	0x52, 0xAA, 0xAA, 0xAB		;24.8kHz



	#define		TESTSTRB	PORTA, 0
	#define		SERIALIN	PORTA, 2
	#define		MKSP		Flags, 0
	#define		FREQFLAG	Flags, 1


	REP1	=	d'60'		;Gives 0.2 second intervals
	REP0	=	0



    org     0

	clrf	INTCON          ;
	goto    startup         ;jump to main code
;------------------ Interrupt routine, get here 4 cycles after TMR overflow --
    org 4
ints      	  ;Interupt service at Fosc / 200.  NO REGISTERS are saved, so must be stopped when changing freq etc,

     	bcf     INTCON , T0IF   ;  every 50 clocks = sampling rate
        bsf	TESTSTRB        ;Spare instruction needed to make up 8 clocks total up to point of reloading TMR0
        movlw   d'228'          ; = NNN:  Timer counts 32 of (Fc / 2)
        movwf   TMR0            ;Timer restarts counting 2 cycles after here.
                                ;To here takes 8 clocks (with the extra instruction)
                                ;NNN = 256 - Count + 8/Prescale  = 256 - 20 + 8/2 = 235


 ;Below here is not time-critical, but must be less than than about 56 clocks total
 ;    to not overrun the whole interrupt service.

	movf	D3, W		;Do this first so there is no jittering due to variable add routine times.
	call	SineTable
	movwf	PORTB

	movf	F0, W
	addwf	D0
	btfss	STATUS, C
	goto	Add1Done
	incf	D1
	btfsc	STATUS, Z
	incf	D2
	btfsc	STATUS, Z
	incf	D3
Add1Done
	movf	F1, W
	addwf	D1
	btfss	STATUS, C
	goto	Add2Done
	incf	D2
	btfsc	STATUS, Z

	incf	D3
Add2Done
	movf	F2, W
	addwf	D2
	btfsc	STATUS, C
	incf	D3	

	movf	F3, W
	addwf	D3			;D0-3 updated ready for next pass through

	incf	IntCount0		;16 bit interrupt counter
	btfsc	STATUS, Z
	incf	IntCount1

	movlw	REP1
	subwf	IntCount1, W
	btfss	STATUS, Z
	goto	OutInt

	movlw	REP0
	subwf	IntCount0, W
	btfss	STATUS, Z
	goto	OutInt

	clrf	IntCount1
	clrf	IntCount2
	movlw	2
	xorwf	Flags		;Toggle the FREQFLAG bit
	
OutInt
	bcf	TESTSTRB		;
	retfie
;=====================================
startup 
	clrf    PORTA       ;Special setup to disable comparator on 'F628
	movlw   7
	movwf   CMCON
	
	bsf     STATUS,RP0      ;ram page 1

        movlw   b'10000000'     ;Internal input, Prescalar /2 no pull-ups
        movwf   OPTION_REG

	movlw   b'00000110'     ;
	movwf   PORTA           ;	A2 is serial in, A1 Link to measure baud rate
	movlw   b'00000000'     ;
	movwf   PORTB           ;

	bcf	STATUS, RP0

	movlw	HIGH(TabContents)
	movwf	PCLATH
	clrf	FSKOffset

	clrf	IntCount0
	clrf	IntCount1

	bsf     INTCON, T0IE   
	bsf	INTCON, GIE
;------------------
MainLoop			;Monitor SERIALIN line for any change
	btfsc	FREQFLAG
	goto	CheckOne

	btfss	MKSP		
	goto	MainLoop	;If unchanged, just carry on looping

	bcf	MKSP
	bcf	INTCON, T0IE	;Stop interrupt while changing freq, as registers aren't saved during service
	movlw	0		;May want to swap this for keying polarity
	movwf	FSKOffset
	goto	ReloadFreq	
;........
CheckOne
	btfsc	MKSP
	goto	MainLoop

	bsf	MKSP
	bcf	INTCON, T0IE	;Stop interrupt while changing freq
	movlw	4		;May want to swap this for keying polarity
	movwf	FSKOffset
	nop			;Equalise the mark/space change glitch length
	nop			; while the interrupt is stopped here
ReloadFreq	
	movlw	LOW(FREQ)
	addwf	FSKOffset, W	

	bsf	STATUS, RP0	;Ram Page 1 to get at EE registers
	movwf	EEADR		; avoid the time wasted by changing bank address for each load

	bsf     EECON1 , RD
	movf    EEDATA , W
	movwf	F3		;F0-3 are in an area of memory common to bank1 and bank0 to

	incf	EEADR
	bsf     EECON1 , RD
	movf    EEDATA , W
	movwf	F2

	incf	EEADR
	bsf     EECON1 , RD
	movf    EEDATA , W
	movwf	F1

	incf	EEADR
	bsf     EECON1 , RD
	movf    EEDATA , W
	movwf	F0		;21 clocks glitch in waveform when freq is changed

	bcf     STATUS , RP0    ;ram page 0	
	bsf	INTCON, T0IE	;Resume interrupt.   T0IF will quite likely have been set in the meantime

	goto	MainLoop
;------------------

;------------------------
GetEE 
    bsf     STATUS,RP0      ;ram page 1
    movwf   EEADR
    bsf     EECON1 , RD
    movf    EEDATA , W
    bcf     STATUS , RP0    ;ram page 0
    return
;-------------------------
StoreEEW          ;Enter with W = EE data,  AutoEEADR = EE address
    bsf     STATUS, RP0
    movwf   EEDATA
    bcf     STATUS, RP0
    movf    AutoEEADR, W
    bsf     STATUS, RP0
    movwf   EEADR    

    bsf     STATUS , RP0    ;All EE registers in Bank 1

    bsf     EECON1 , WREN   ;Enable EEprom writing
    movlw   0x55
    movwf   EECON2
    movlw   0xAA
    movwf   EECON2
    bsf     EECON1 , WR

    bcf     STATUS, RP0     ;PIR1 register is in bank 0
EEwaitW
   
    btfss   PIR1 , EEIF
    goto    EEwaitW
    bcf     PIR1 , EEIF       ;has to be cleared manually

    bsf     STATUS, RP0
    bcf     EECON1 , WR
    bcf     STATUS , RP0

    return

;---------------------

  org 0x6FF
SineTable
		;Adjust so TabContents falls EXACTLY on the next page boundary
	addwf	PCL
TabContents
    dt    0x80,0x83,0x86,0x89,0x8C,0x8F,0x92,0x95
    dt    0x98,0x9C,0x9F,0xA2,0xA5,0xA8,0xAB,0xAE
    dt    0xB0,0xB3,0xB6,0xB9,0xBC,0xBF,0xC1,0xC4
    dt    0xC7,0xC9,0xCC,0xCE,0xD1,0xD3,0xD5,0xD8
    dt    0xDA,0xDC,0xDE,0xE0,0xE2,0xE4,0xE6,0xE8
    dt    0xEA,0xEC,0xED,0xEF,0xF0,0xF2,0xF3,0xF5
    dt    0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFC
    dt    0xFD,0xFE,0xFE,0xFF,0xFF,0xFF,0xFF,0xFF
    dt    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFE,0xFE
    dt    0xFD,0xFC,0xFC,0xFB,0xFA,0xF9,0xF8,0xF7
    dt    0xF6,0xF5,0xF3,0xF2,0xF0,0xEF,0xED,0xEC
    dt    0xEA,0xE8,0xE6,0xE4,0xE2,0xE0,0xDE,0xDC
    dt    0xDA,0xD8,0xD5,0xD3,0xD1,0xCE,0xCC,0xC9
    dt    0xC7,0xC4,0xC1,0xBF,0xBC,0xB9,0xB6,0xB3
    dt    0xB0,0xAE,0xAB,0xA8,0xA5,0xA2,0x9F,0x9C
    dt    0x98,0x95,0x92,0x8F,0x8C,0x89,0x86,0x83
    dt    0x7F,0x7C,0x79,0x76,0x73,0x70,0x6D,0x6A
    dt    0x67,0x63,0x60,0x5D,0x5A,0x57,0x54,0x51
    dt    0x4F,0x4C,0x49,0x46,0x43,0x40,0x3E,0x3B
    dt    0x38,0x36,0x33,0x31,0x2E,0x2C,0x2A,0x27
    dt    0x25,0x23,0x21,0x1F,0x1D,0x1B,0x19,0x17
    dt    0x15,0x13,0x12,0x10,0x0F,0x0D,0x0C,0x0A
    dt    0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x03
    dt    0x02,0x01,0x01,0x00,0x00,0x00,0x00,0x00
    dt    0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01
    dt    0x02,0x03,0x03,0x04,0x05,0x06,0x07,0x08
    dt    0x09,0x0A,0x0C,0x0D,0x0F,0x10,0x12,0x13
    dt    0x15,0x17,0x19,0x1B,0x1D,0x1F,0x21,0x23
    dt    0x25,0x27,0x2A,0x2C,0x2E,0x31,0x33,0x36
    dt    0x38,0x3B,0x3E,0x40,0x43,0x46,0x49,0x4C
    dt    0x4F,0x51,0x54,0x57,0x5A,0x5D,0x60,0x63
    dt    0x67,0x6A,0x6D,0x70,0x73,0x76,0x79,0x7C

;--------------------------------
    Cblock 0x20
	Temp
	Flags
	DelCount
	Counter
	IntCount0, IntCount1, IntCount2
	AutoEEADR
	FSKOffset
	D0, D1, D2, D3
	HoldF0, HoldF1, HoldF2, HoldF3, 
    endc

    Cblock 0x70
	F0, F1, F2, F3		;These are in an area common to bank 0 and bank 1
    endc

	end















